/***********************************************************************
*   Quick File Manager Panel Applet
*
*   Copyright (C) 2004 M.K.Srikant
*   Author: M.K.Srikant <srix_mk@yahoo.co.in>
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*************************************************************************/

/************************************************************************
 * Revision History:
 * 
 * v1.2: released 15-Nov-2006
 *    - added icons for menu items based on mime type
 *    - minimal code comments added
 *    - applet icon added in About box (instead of default About icon)
 *    - 
 *
 * v1.1: released 19-Sep-2006
************************************************************************/

#include <stdio.h>
#include <time.h>
#include <gnome.h>
#include <panel-applet.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime.h>
#include "panelfm.xpm"

#define     MAX_BUF   1024
#define     MEM_SIZE  1024000
#define     VERSION_STR    "1.2"

static void about_cb (PanelApplet * widget, gpointer data);
static char buf[MAX_BUF];
static char *menubuf, *menubufptr, *cursel;
static GtkWidget *top_menu, *context_menu;

static const char menu_xml[] =
   "<popup name=\"button3\">\n"
   "   <menuitem name=\"About Item\" verb=\"GNAbout\" label=\"About\"\n"
   "             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
   "</popup>\n";

static const BonoboUIVerb menu_verbs[] = {
   BONOBO_UI_VERB ("GNAbout", about_cb),
   BONOBO_UI_VERB_END
};


/******************************************************************/
void dbg (char *msg)
{
   GtkWidget *w;

   w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
                               GTK_BUTTONS_OK, msg);
   gtk_dialog_run (GTK_DIALOG (w));
   gtk_widget_destroy (w);
}


/******************************************************************
 * Confirmation dialog on choosing to delete a file
******************************************************************/
int query (char *msg)
{
   GtkWidget *w;
   int ret;

   w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                               GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
                               msg);
   ret = gtk_dialog_run (GTK_DIALOG (w));
   gtk_widget_destroy (w);

   return (ret);
}


/******************************************************************
 * Move selected file to trash can (NOT immediate delete) - TODO
******************************************************************/
static gint trash_file (GtkWidget * w, gpointer * val)
{
   return (0);
}


/******************************************************************
 * Delete file after confirmation (immediate delete)
******************************************************************/
static gint delete_file (GtkWidget * w, gpointer * val)
{
   int res;

   sprintf (buf, "Are you sure you want to delete %s?", cursel);
   res = query (buf);
   if (res == GTK_RESPONSE_YES)
      unlink (cursel);
   return (0);
}


/******************************************************************/
static gint open_terminal (GtkWidget * w, gpointer * val)
{
   sprintf (buf, "gnome-terminal --working-directory=%s &", (char *)val);
   system (buf);
   return (0);
}


/******************************************************************/
static gint display_file_dir (GtkWidget * w, gpointer * val)
{
   GdkEvent *ev;

   ev = gtk_get_current_event ();
   if (ev->button.button == 1)
   {
      sprintf (buf, "file://%s", (char *)val);
      gnome_vfs_url_show (buf);
   }
   else if (ev->button.button == 3)
   {
      cursel = (char *) val;
      gtk_menu_popup (GTK_MENU (context_menu), NULL, NULL, NULL, NULL,
                      0, gtk_get_current_event_time ());
   }
   gdk_event_free (ev);

   return (0);
}


/******************************************************************/
gint cmp_finfo (gpointer a, gpointer b)
{
   GnomeVFSFileInfo *fa, *fb;

   fa = (GnomeVFSFileInfo *) a;
   fb = (GnomeVFSFileInfo *) b;
   return (strcmp (fa->name, fb->name));
}


/******************************************************************
 * When top level menu item is activated, create and display menu containing
 * list of files and dirs. contained in the chosen path. Also add standard
 * "Open" and "Open in Terminal" options for dir. entries, and add signal
 * handlers for activation of any menu entry
******************************************************************/
static gint make_fm_menu (GtkWidget * menu, gpointer * selection)
{
   char *p;
   GtkWidget *smenu, *menu_item, *tmp;
   GnomeVFSFileInfo *finfo;
   GList *f, *lst = NULL;

   if (gnome_vfs_directory_list_load (&lst, (gchar *) selection,
                                      GNOME_VFS_FILE_INFO_DEFAULT |
                                      GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
                                      GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE)
       == GNOME_VFS_OK)
   {
      finfo = gnome_vfs_file_info_new ();

      smenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
      if (!smenu)               // first-time menu popping up
         smenu = menu;
      else                      // destroy old sub-menu and re-create it
      {
         gtk_widget_ref (smenu);
         gtk_menu_item_remove_submenu (GTK_MENU_ITEM (menu));
         gtk_widget_destroy (smenu);
         smenu = gtk_menu_new ();
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), smenu);
      }
      menu_item = gtk_menu_item_new_with_label ("Open");
      g_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (display_file_dir),
                        (gpointer) selection);
      gtk_menu_shell_append (GTK_MENU_SHELL (smenu), menu_item);
      gtk_widget_show (menu_item);

      menu_item = gtk_menu_item_new_with_label ("Open in Terminal");
      g_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (open_terminal),
                        (gpointer) selection);
      gtk_menu_shell_append (GTK_MENU_SHELL (smenu), menu_item);
      gtk_widget_show (menu_item);

      menu_item = gtk_separator_menu_item_new ();
      gtk_menu_shell_append (GTK_MENU_SHELL (smenu), menu_item);
      gtk_widget_show (menu_item);

      f = g_list_sort (lst, (GCompareFunc) cmp_finfo);
      while (f)
      {
         finfo = f->data;
         f = f->next;
         if (finfo->name[0] == '.') // don't show dot-files
            continue;
         sprintf (buf, "%s/%s", (char *)selection, finfo->name);
         strcpy (menubufptr, buf);
         p = menubufptr;
         menubufptr += strlen (buf) + 1;

            /***
             * create submenu for directory entries
             ***/
         if (finfo->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
         {
            menu_item = gtk_image_menu_item_new_with_label (finfo->name);
            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
                         gtk_image_new_from_icon_name
                         (gnome_icon_lookup (gtk_icon_theme_get_default (), 
                                    NULL, p, NULL, finfo,
                                    finfo->mime_type,
                                    GNOME_ICON_LOOKUP_FLAGS_EMBEDDING_TEXT |
                                    GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES |
                                    GNOME_ICON_LOOKUP_FLAGS_ALLOW_SVG_AS_THEMSELVES,
                                    NULL), 
                          GTK_ICON_SIZE_MENU));
            g_signal_connect (GTK_OBJECT (menu_item), "activate",
                              GTK_SIGNAL_FUNC (make_fm_menu),
                              (gpointer) p);
            gtk_menu_shell_append (GTK_MENU_SHELL (smenu), menu_item);
            tmp = gtk_menu_new ();
            gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), tmp);
            gtk_widget_show (tmp);
         }
         else                   // normal files are just menu items
         {
            menu_item = gtk_image_menu_item_new_with_label (finfo->name);
            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
                         gtk_image_new_from_icon_name
                         (gnome_icon_lookup (gtk_icon_theme_get_default (), 
                                    NULL, p, NULL, finfo,
                                    finfo->mime_type,
                                    GNOME_ICON_LOOKUP_FLAGS_EMBEDDING_TEXT |
                                    GNOME_ICON_LOOKUP_FLAGS_SHOW_SMALL_IMAGES_AS_THEMSELVES |
                                    GNOME_ICON_LOOKUP_FLAGS_ALLOW_SVG_AS_THEMSELVES,
                                    NULL), 
                          GTK_ICON_SIZE_MENU));
            gtk_widget_add_events (menu_item, GDK_KEY_PRESS_MASK);
            g_signal_connect_after (GTK_OBJECT (menu_item), "activate",
                                    GTK_SIGNAL_FUNC (display_file_dir),
                                    (gpointer) p);
            gtk_menu_shell_append (GTK_MENU_SHELL (smenu), menu_item);
         }
         gtk_widget_show (smenu);
         gtk_widget_show (menu_item);
      }
   }
   gtk_widget_show (menu);

   return (1);
}


/******************************************************************
 * Create top level menu containing home path or list of paths defined in
 * $HOME/.panelfmrc, and attach signal handler for menu item activation
******************************************************************/
static gint make_top_menu ()
{
   FILE *fp;
   GtkWidget *menu_item, *tmp;
   int len;
   char *p, *homedir;

   top_menu = gtk_menu_new ();

   homedir = getenv ("HOME");
   sprintf (buf, "%s/.panelfmrc", homedir);
   fp = fopen (buf, "r");
   if (!fp)
   {
      p = strdup (homedir);
      menu_item = gtk_menu_item_new_with_label (p);
      g_signal_connect (GTK_OBJECT (menu_item), "activate",
                        GTK_SIGNAL_FUNC (make_fm_menu), (gpointer) p);
      gtk_menu_shell_append (GTK_MENU_SHELL (top_menu), menu_item);
      gtk_widget_show (menu_item);
      tmp = gtk_menu_new ();
      gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), tmp);
      gtk_widget_show (tmp);
   }
   else
   {
      while (fgets (buf, 128, fp))
      {
         len = strlen (buf);
         if (buf[len - 1] == '\n')
            buf[len - 1] = '\0';
         p = strdup (buf);
         menu_item = gtk_menu_item_new_with_label (buf);
         g_signal_connect (GTK_OBJECT (menu_item), "activate",
                           GTK_SIGNAL_FUNC (make_fm_menu), (gpointer) p);
         gtk_menu_shell_append (GTK_MENU_SHELL (top_menu), menu_item);
         gtk_widget_show (menu_item);
         tmp = gtk_menu_new ();
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), tmp);
         gtk_widget_show (tmp);
      }
      fclose (fp);
   }

   return (1);
}


/******************************************************************
 * Create top level menu (by calling "make_top_menu") and display it
******************************************************************/
static gint show_top_menu (GtkWidget * widget, GdkEventButton * event)
{
   if (event->button != 1)
      return (0);

   make_top_menu ();
   memset (menubuf, '\0', MEM_SIZE);
   menubufptr = menubuf;

   gtk_menu_popup (GTK_MENU (top_menu), NULL, NULL, NULL, NULL,
                   0, gtk_get_current_event_time ());

   return (1);
}


/******************************************************************
 * Create context menu that will be shown when right mouse button clicked on
 * a file name in any sub menu
******************************************************************/
static gint make_context_menu ()
{
   GtkWidget *menu_item;

   context_menu = gtk_menu_new ();
   /*
    * menu_item = gtk_menu_item_new_with_label ("Move to Trash");
    * g_signal_connect (GTK_OBJECT (menu_item), "activate",
    * GTK_SIGNAL_FUNC (trash_file), NULL);
    * gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
    * gtk_widget_show (menu_item);
    */

   menu_item = gtk_menu_item_new_with_label ("Delete");
   g_signal_connect (GTK_OBJECT (menu_item), "activate",
                     GTK_SIGNAL_FUNC (delete_file), NULL);
   gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
   gtk_widget_show (menu_item);

   return (0);
}


/************************************************************/
static void about_cb (PanelApplet * widget, gpointer data)
{
   GtkWidget *w;
   GdkPixbuf *pixbuf;
   static const gchar *s[] =
      { "M.K.Srikant <srix_mk@yahoo.co.in>", NULL };

   pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **) panelfm_xpm);
   w = gnome_about_new ("PanelFM", VERSION_STR,
                        "Copyright (c) M.K.Srikant 2004",
                        "A Quick File Manager Applet.\nReleased under the GNU Public License.",
                        s, NULL, NULL, pixbuf);
   gtk_widget_show (w);
}


/************************************************************
 * Set up applet to show icon on panel and attach signal handler for when
 * mouse is clicked on the applet icon
************************************************************/
static GtkWidget *show_on_panel (PanelApplet * applet)
{
   GtkWidget *img, *eventbox;
   GdkPixbuf *pixbuf, *tmp;
   guint size;

   chdir (getenv ("HOME"));

   gnome_vfs_init ();

   menubuf = malloc (MEM_SIZE);
   // make_top_menu ();
   make_context_menu ();

   size = panel_applet_get_size (applet);
   eventbox = gtk_event_box_new ();
   pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **) panelfm_xpm);
   tmp =
      gdk_pixbuf_scale_simple (pixbuf, size, size, GDK_INTERP_BILINEAR);
   img = gtk_image_new_from_pixbuf (tmp);
   gtk_container_add (GTK_CONTAINER (eventbox), img);
   g_signal_connect (GTK_OBJECT (eventbox), "button_release_event",
                     GTK_SIGNAL_FUNC (show_top_menu), NULL);
   gtk_container_add (GTK_CONTAINER (applet), eventbox);

   gtk_widget_show_all (GTK_WIDGET (applet));
   return (img);
}


/************************************************************
 * Start here!
************************************************************/
int panelfm_main (PanelApplet * app, const gchar * iid, gpointer data)
{
   GtkWidget *onpanel;
   GtkTooltips *tip;

   panel_applet_setup_menu (app, menu_xml, menu_verbs, (gpointer) app);

   onpanel = show_on_panel (app);
   tip = gtk_tooltips_new ();                                                                 
   gtk_tooltips_set_tip (tip, (GtkWidget *)app, 
                        "Browse through files and directories", NULL); 
   gtk_tooltips_enable (tip);
   gtk_widget_show (onpanel);

   return (TRUE);
}

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:PanelFM_Factory",
                             PANEL_TYPE_APPLET,
                             "Quick File Manager Applet",
                             VERSION_STR, panelfm_main, NULL);

